==> review_0.1.tar.gz: review/src/com/monblocnotes/review/ManagePairsDialog.java <==
/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

package com.monblocnotes.review;

import java.util.ArrayList;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public class ManagePairsDialog extends Dialog {
	
	static final int INPUT_FIELDS_LENGTH = 200;
	static final int TABLE_HEIGHT = 300;
	static final int BUTTON_HEIGHT = 30;
	static final int BUTTON_LENGTH = 100;
	
	private Shell shell = null;
	private Shell parent = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	private DataAccess dataAccess = null;
	
	private Composite fieldsAndButtonsC = null; 
	private Composite fieldsC = null;
	private Composite buttonsC = null;
	private Table pairsTbl = null;
	private Text wordATxt = null;
	private Text wordBTxt = null;
	private Button addBtn = null;
	private Button editBtn = null;
	private Button deleteBtn = null;
	private Button validateBtn = null;
	private Button cancelBtn = null;
	private Button exitBtn = null;
	
	// PreviousWordA is used when an edit operation is requested.
	private String previousWordA = null;
	// Same for previousWordB.
	private String previousWordB = null;
	
	// Used in validate event processing, to know whether we were adding a pair, or
	// editing one.
	private boolean currentlyAdding = false;

	public ManagePairsDialog(Shell parent, int style) {
		super(parent);	
	}
	
	public ManagePairsDialog(Shell parent) {
		super(parent);
	}

	static public void refreshPairsTable(DataAccess dataAccess, Table pairsTbl) {
		// Remove all displayed lessons.
		pairsTbl.removeAll();
		// And display list of existing lessons.
		ArrayList<PairInfo> pairs = dataAccess.getPairs();
		int nb = pairs.size();
		String [] data = new String[2];
		for (int i = 0; i < nb; i++) {
			data[0] = pairs.get(i).getWordLanguageA();
			data[1] = pairs.get(i).getWordLanguageB();
			TableItem item = new TableItem(pairsTbl, SWT.NONE);
			item.setText(data);
		}
	}
	
	public void configureOnAdd() {
		currentlyAdding = true;
		fieldsC.setVisible(true);
		addBtn.setVisible(false);
		editBtn.setVisible(false);
		deleteBtn.setVisible(false);
		validateBtn.setVisible(true);
		cancelBtn.setVisible(true);
		exitBtn.setVisible(false);
	}
	
	public void configureOnEdit() {
		currentlyAdding = false;
		fieldsC.setVisible(true);
		addBtn.setVisible(false);
		editBtn.setVisible(false);
		deleteBtn.setVisible(false);
		validateBtn.setVisible(true);
		cancelBtn.setVisible(true);
		exitBtn.setVisible(false);
	}
	
	public void configureOnValidate() {
		currentlyAdding = false;
		fieldsC.setVisible(false);
		addBtn.setVisible(true);
		editBtn.setVisible(true);
		deleteBtn.setVisible(true);
		validateBtn.setVisible(false);
		cancelBtn.setVisible(false);
		exitBtn.setVisible(true);
	}
	
	public void configureOnCancel() {
		// TODO
		currentlyAdding = false;
		fieldsC.setVisible(false);
		addBtn.setVisible(true);
		editBtn.setVisible(true);
		deleteBtn.setVisible(true);
		validateBtn.setVisible(false);
		cancelBtn.setVisible(false);
		exitBtn.setVisible(true);
	}
	
	public void prepareDialog(DataAccess dataAccessIn, ArrayList<PairInfo> lessons){

		final String[] columnNames = {resReview.getString("MWW_word_language_A"),
				                      resReview.getString("MWW_word_language_B")};

		dataAccess = dataAccessIn;
		parent = getParent(); 
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(resReview.getString("MWW_title"));
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		
		// Create a layout with 2 columns. Columns are not forced to have the same width.
		// Left column will contain the pairs table.
		// Right column will contain input fields and buttons.
		GridLayout pairsGL = new GridLayout(2, false);
		shell.setLayout(pairsGL);
		
		// Display pairs table.
		pairsTbl = new Table(shell, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION);
		pairsTbl.setHeaderVisible(true);
		pairsTbl.setLinesVisible(true);
		//
		for(int i = 0; i < columnNames.length; i++) {
			TableColumn column = new TableColumn(pairsTbl, SWT.NONE);
			column.setText(columnNames[i]);
			column.setWidth(INPUT_FIELDS_LENGTH);
		}
		
		// Force a given height, for the table.
		GridData pairsGD = new GridData(SWT.FILL, SWT.FILL, true, true);
		pairsGD.heightHint = TABLE_HEIGHT;
		pairsTbl.setLayoutData(pairsGD);
		// Display data.
		refreshPairsTable(dataAccess, pairsTbl);
		
		// Add a composite, that will contain input fields and buttons.
		// It contains one column. Two rows will be used. The upper one for the
		// input fields, the lower one for the buttons.
		fieldsAndButtonsC = new Composite(shell, SWT.NONE);
		GridLayout fieldsAndButtonsGL = new GridLayout(1, true);
		fieldsAndButtonsC.setLayout(fieldsAndButtonsGL);
		
		// Add a new composite, for input fields.
		fieldsC = new Composite(fieldsAndButtonsC, SWT.BORDER);
		GridLayout fieldsGL = new GridLayout(2, true);
		fieldsC.setLayout(fieldsGL);
		// Add input fields.
		// Word in language A.
		Label wordALbl = new Label(fieldsC, SWT.RIGHT);
		wordALbl.setText(resReview.getString("MWW_word_in_language_A"));	
		wordATxt = new Text(fieldsC, SWT.BORDER);
		// Force length of input text.
		GridData wordAGD = new GridData();
		wordAGD.widthHint = INPUT_FIELDS_LENGTH;
		wordATxt.setLayoutData(wordAGD);
		// Word in language B.
		Label wordBLbl = new Label(fieldsC, SWT.RIGHT);
		wordBLbl.setText(resReview.getString("MWW_word_in_language_B"));
		wordBTxt = new Text(fieldsC, SWT.BORDER);
		GridData wordBGD = new GridData();
		wordBGD.widthHint = INPUT_FIELDS_LENGTH;
		wordBTxt.setLayoutData(wordBGD);
		// At startup, this composite is not visible.
		fieldsC.setVisible(false);
		
		// Add a new composite, that will allow for centering buttons.
		buttonsC = new Composite(fieldsAndButtonsC, SWT.NONE);
		// Give our requirements for positioning of buttons.
		GridData buttonsGD = new GridData();
		buttonsGD.horizontalAlignment = SWT.CENTER;
		buttonsC.setLayoutData(buttonsGD);
		// And apply a row layout.
		RowLayout buttonsRL = new RowLayout(SWT.VERTICAL);
		buttonsC.setLayout(buttonsRL);
		
		// Add buttons.
		// Add button.
	    addBtn = new Button(buttonsC, SWT.PUSH);
		addBtn.setText(resReview.getString("MWW_add"));
		addBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		addBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				// Configure UI.
				configureOnAdd();
				// Put the cursor at the right place.
				wordATxt.setFocus();
				// Next step for the user: enter values and click on Validate or Cancel.
			}
		});
		// Edit button.
		editBtn = new Button(buttonsC, SWT.PUSH);
		editBtn.setText(resReview.getString("MWW_edit"));
		editBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		editBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (pairsTbl.getSelectionCount() > 0) {
					// One pair is selected, display it.
					TableItem [] pairTI = pairsTbl.getSelection();
					String wordA = pairTI[0].getText(0);
					previousWordA = wordA;
					String wordB = pairTI[0].getText(1);
					previousWordB = wordB;
					wordATxt.setText(wordA);
					wordBTxt.setText(wordB);
					// Configure UI.
					configureOnEdit();
					// Put the cursor at the right place.
					wordATxt.setFocus();
					// Next step for the user: modify values and click on Validate or Cancel.
				}
				else {
					MessageBox box = new MessageBox(shell, SWT.NONE);
					box.setText(resReview.getString("MWW_warning"));
					box.setMessage(resReview.getString("MWW_select_a_pair"));
					box.open();										
				}
			}
		});
		// Delete button.
		deleteBtn = new Button(buttonsC, SWT.PUSH);
		deleteBtn.setText(resReview.getString("MWW_delete"));
		deleteBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		deleteBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (pairsTbl.getSelectionCount() > 0) {
					TableItem [] pairTI = pairsTbl.getSelection();
					String wordA = pairTI[0].getText(0);
					String wordB = pairTI[0].getText(1);
					ErrorInfo ei = dataAccess.deletePair(wordA, wordB);
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MessageBox box = new MessageBox(shell, SWT.NONE);
						box.setText(resReview.getString("MWW_error"));
						box.setMessage(ei.getDescription());
						box.open();
						// Stay in the same state.
						return;
					}
					// If no error, refresh pairs table.
					refreshPairsTable(dataAccess, pairsTbl);
				} else {
					MessageBox box = new MessageBox(shell, SWT.NONE);
					box.setText(resReview.getString("WRN_warning"));
					box.setMessage(resReview.getString("MWW_select_a_pair"));
					box.open();					
				}
			}
		});
		// Validate button.
		validateBtn = new Button(buttonsC, SWT.PUSH);
		validateBtn.setText(resReview.getString("MWW_validate"));
		validateBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		validateBtn.setVisible(false);
		validateBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (currentlyAdding) {
					// Validate entered fields and save them if OK.
					ErrorInfo ei = dataAccess.addPair(wordATxt.getText(),
							                          wordBTxt.getText());
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MessageBox box = new MessageBox(shell, SWT.NONE);
						box.setText(resReview.getString("ERR_error"));
						box.setMessage(ei.getDescription());
						box.open();
						// Stay in same state. And pre-position the cursor.
						switch (ei.getValue()) {
						case DataAccess.DA_WORDTOOLONG :
							// TODO: differentiate wordA and wordB.
							wordATxt.setFocus();
							break;
						default :		
							wordATxt.setFocus();
						}
						return;
					}
					// If no error, refresh lessons table.
					refreshPairsTable(dataAccess, pairsTbl);
				} else {
					// Update existing lesson.
					ErrorInfo ei = dataAccess.updatePair(previousWordA,
							                             previousWordB,
							                             wordATxt.getText(),
					                                     wordBTxt.getText());
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MessageBox box = new MessageBox(shell, SWT.NONE);
						box.setText(resReview.getString("ERR_error"));
						box.setMessage(ei.getDescription());
						box.open();
						// Stay in same state. And pre-position the cursor.
						switch (ei.getValue()) {
						case DataAccess.DA_WORDTOOLONG :
							// TODO: differentiate wordA and wordB.
							wordATxt.setFocus();
							break;
						default :		
							wordATxt.setFocus();
						}
						return;
					}
					// If no error, refresh lessons table.
					refreshPairsTable(dataAccess, pairsTbl);
				}
				//
				configureOnValidate();
			}
		});
		
		cancelBtn = new Button(buttonsC, SWT.PUSH);
		cancelBtn.setText(resReview.getString("MWW_cancel"));
		cancelBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		cancelBtn.setVisible(false);
		cancelBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				configureOnCancel();
			}
		});
		
		exitBtn = new Button(buttonsC, SWT.PUSH);
		exitBtn.setText(resReview.getString("MWW_exit"));
		exitBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}
		});

		
	}
	
	public void open() {
		
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}

	}


}

==> review-v0.4.tar.gz: review/src/com/monblocnotes/review/ManagePairsDialog.java <==
/*******************************************************************************
 * Copyright (c) 2009 Pascal Bodin
 * pascal dot bodin at monblocnotes dot com

This file is part of Review.

Review is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Review is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Review.  If not, see <http://www.gnu.org/licenses/>.

*******************************************************************************/

package com.monblocnotes.review;

import java.util.ArrayList;
import java.util.ResourceBundle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public class ManagePairsDialog extends Dialog {
	
	private static final int INPUT_FIELDS_LENGTH = 200;
	private static final int TABLE_HEIGHT = 300;
	private static final int BUTTON_HEIGHT = 30;
	private static final int BUTTON_LENGTH = 100;
	
	private Shell shell = null;
	private Shell parent = null;
	private static ResourceBundle resReview = ResourceBundle.getBundle("review");
	private DataAccess dataAccess = null;
	
	private Composite fieldsAndButtonsC = null; 
	private Composite fieldsC = null;
	private Composite buttonsC = null;
	private Table pairsTbl = null;
	private Text wordATxt = null;
	private Text wordBTxt = null;
	private Button addBtn = null;
	private Button editBtn = null;
	private Button deleteBtn = null;
	private Button validateBtn = null;
	private Button cancelBtn = null;
	private Button exitBtn = null;
	
	// PreviousWordA is used when an edit operation is requested.
	private String previousWordA = null;
	// Same for previousWordB.
	private String previousWordB = null;
	
	// Used in validate event processing, to know whether we were adding a pair, or
	// editing one.
	private boolean currentlyAdding = false;

	public ManagePairsDialog(Shell parent, int style) {
		super(parent);	
	}
	
	public ManagePairsDialog(Shell parent) {
		super(parent);
	}

	static public void refreshPairsTable(DataAccess dataAccess, Table pairsTbl) {
		// Remove all displayed lessons.
		pairsTbl.removeAll();
		// And display list of existing lessons.
		ArrayList<PairInfo> pairs = dataAccess.getPairs();
		int nb = pairs.size();
		String [] data = new String[2];
		for (int i = 0; i < nb; i++) {
			data[0] = pairs.get(i).getWordLanguageA();
			data[1] = pairs.get(i).getWordLanguageB();
			TableItem item = new TableItem(pairsTbl, SWT.NONE);
			item.setText(data);
		}
	}
	
	public void configureOnAdd() {
		currentlyAdding = true;
		fieldsC.setVisible(true);
		addBtn.setVisible(false);
		editBtn.setVisible(false);
		deleteBtn.setVisible(false);
		validateBtn.setVisible(true);
		cancelBtn.setVisible(true);
		exitBtn.setVisible(false);
	}
	
	public void configureOnEdit() {
		currentlyAdding = false;
		fieldsC.setVisible(true);
		addBtn.setVisible(false);
		editBtn.setVisible(false);
		deleteBtn.setVisible(false);
		validateBtn.setVisible(true);
		cancelBtn.setVisible(true);
		exitBtn.setVisible(false);
	}
	
	public void configureOnValidate() {
		currentlyAdding = false;
		fieldsC.setVisible(false);
		addBtn.setVisible(true);
		editBtn.setVisible(true);
		deleteBtn.setVisible(true);
		validateBtn.setVisible(false);
		cancelBtn.setVisible(false);
		exitBtn.setVisible(true);
	}
	
	public void configureOnCancel() {
		currentlyAdding = false;
		fieldsC.setVisible(false);
		addBtn.setVisible(true);
		editBtn.setVisible(true);
		deleteBtn.setVisible(true);
		validateBtn.setVisible(false);
		cancelBtn.setVisible(false);
		exitBtn.setVisible(true);
	}
	
	public void prepareDialog(DataAccess dataAccessIn, ArrayList<PairInfo> lessons){

		final String[] columnNames = {resReview.getString("MWW_word_language_A"),
				                      resReview.getString("MWW_word_language_B")};

		dataAccess = dataAccessIn;
		parent = getParent(); 
		shell = new Shell(parent, SWT.DIALOG_TRIM | SWT.APPLICATION_MODAL);
		shell.setText(resReview.getString("MWW_title"));
		shell.addShellListener(new ShellAdapter(){
			public void shellClosed(ShellEvent e) {
				shell.dispose();
			}
		});
		
		// Create a layout with 2 columns. Columns are not forced to have the same width.
		// Left column will contain the pairs table.
		// Right column will contain input fields and buttons.
		GridLayout pairsGL = new GridLayout(2, false);
		shell.setLayout(pairsGL);
		
		// Display pairs table.
		pairsTbl = new Table(shell, SWT.SINGLE | SWT.BORDER | SWT.FULL_SELECTION);
		pairsTbl.setHeaderVisible(true);
		pairsTbl.setLinesVisible(true);
		//
		for(int i = 0; i < columnNames.length; i++) {
			TableColumn column = new TableColumn(pairsTbl, SWT.NONE);
			column.setText(columnNames[i]);
			column.setWidth(INPUT_FIELDS_LENGTH);
		}
		
		// Force a given height, for the table.
		GridData pairsGD = new GridData(SWT.FILL, SWT.FILL, true, true);
		pairsGD.heightHint = TABLE_HEIGHT;
		pairsTbl.setLayoutData(pairsGD);
		// Display data.
		refreshPairsTable(dataAccess, pairsTbl);
		
		// Add a composite, that will contain input fields and buttons.
		// It contains one column. Two rows will be used. The upper one for the
		// input fields, the lower one for the buttons.
		fieldsAndButtonsC = new Composite(shell, SWT.NONE);
		GridLayout fieldsAndButtonsGL = new GridLayout(1, true);
		fieldsAndButtonsC.setLayout(fieldsAndButtonsGL);
		
		// Add a new composite, for input fields.
		fieldsC = new Composite(fieldsAndButtonsC, SWT.BORDER);
		GridLayout fieldsGL = new GridLayout(2, true);
		fieldsC.setLayout(fieldsGL);
		// Add input fields.
		// Word in language A.
		Label wordALbl = new Label(fieldsC, SWT.RIGHT);
		wordALbl.setText(resReview.getString("MWW_word_in_language_A"));	
		wordATxt = new Text(fieldsC, SWT.BORDER);
		// Force length of input text.
		GridData wordAGD = new GridData();
		wordAGD.widthHint = INPUT_FIELDS_LENGTH;
		wordATxt.setLayoutData(wordAGD);
		// Word in language B.
		Label wordBLbl = new Label(fieldsC, SWT.RIGHT);
		wordBLbl.setText(resReview.getString("MWW_word_in_language_B"));
		wordBTxt = new Text(fieldsC, SWT.BORDER);
		GridData wordBGD = new GridData();
		wordBGD.widthHint = INPUT_FIELDS_LENGTH;
		wordBTxt.setLayoutData(wordBGD);
		// At startup, this composite is not visible.
		fieldsC.setVisible(false);
		
		// Add a new composite, that will allow for centering buttons.
		buttonsC = new Composite(fieldsAndButtonsC, SWT.NONE);
		// Give our requirements for positioning of buttons.
		GridData buttonsGD = new GridData();
		buttonsGD.horizontalAlignment = SWT.CENTER;
		buttonsC.setLayoutData(buttonsGD);
		// And apply a row layout.
		RowLayout buttonsRL = new RowLayout(SWT.VERTICAL);
		buttonsC.setLayout(buttonsRL);
		
		// Add buttons.
		// Add button.
	    addBtn = new Button(buttonsC, SWT.PUSH);
		addBtn.setText(resReview.getString("MWW_add"));
		addBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		addBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				// Configure UI.
				configureOnAdd();
				// Put the cursor at the right place.
				wordATxt.setFocus();
				// Next step for the user: enter values and click on Validate or Cancel.
			}
		});
		// Edit button.
		editBtn = new Button(buttonsC, SWT.PUSH);
		editBtn.setText(resReview.getString("MWW_edit"));
		editBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		editBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (pairsTbl.getSelectionCount() > 0) {
					// One pair is selected, display it.
					TableItem [] pairTI = pairsTbl.getSelection();
					String wordA = pairTI[0].getText(0);
					previousWordA = wordA;
					String wordB = pairTI[0].getText(1);
					previousWordB = wordB;
					wordATxt.setText(wordA);
					wordBTxt.setText(wordB);
					// Configure UI.
					configureOnEdit();
					// Put the cursor at the right place.
					wordATxt.setFocus();
					// Next step for the user: modify values and click on Validate or Cancel.
				}
				else {
					MyMessageBox messageBox = new MyMessageBox(shell, SWT.OK);
					messageBox.setText(resReview.getString("MWW_warning"));
					messageBox.setMessage(resReview.getString("MWW_select_a_pair"));
					messageBox.open();										
				}
			}
		});
		// Delete button.
		deleteBtn = new Button(buttonsC, SWT.PUSH);
		deleteBtn.setText(resReview.getString("MWW_delete"));
		deleteBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		deleteBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (pairsTbl.getSelectionCount() > 0) {
					TableItem [] pairTI = pairsTbl.getSelection();
					String wordA = pairTI[0].getText(0);
					String wordB = pairTI[0].getText(1);
					ErrorInfo ei = dataAccess.deletePair(wordA, wordB);
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MyMessageBox messageBox = new MyMessageBox(shell, SWT.OK);
						messageBox.setText(resReview.getString("MWW_error"));
						messageBox.setMessage(ei.getDescription());
						messageBox.open();
						// Stay in the same state.
						return;
					}
					// If no error, refresh pairs table.
					refreshPairsTable(dataAccess, pairsTbl);
				} else {
					MyMessageBox messageBox = new MyMessageBox(shell, SWT.OK);
					messageBox.setText(resReview.getString("MWW_warning"));
					messageBox.setMessage(resReview.getString("MWW_select_a_pair"));
					messageBox.open();					
				}
			}
		});
		// Validate button.
		validateBtn = new Button(buttonsC, SWT.PUSH);
		validateBtn.setText(resReview.getString("MWW_validate"));
		validateBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		validateBtn.setVisible(false);
		validateBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				if (currentlyAdding) {
					// Validate entered fields and save them if OK.
					ErrorInfo ei = dataAccess.addPair(wordATxt.getText(),
							                          wordBTxt.getText());
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MyMessageBox messageBox = new MyMessageBox(shell, SWT.OK);
						messageBox.setText(resReview.getString("ERR_error"));
						messageBox.setMessage(ei.getDescription());
						messageBox.open();
						// Stay in same state. And pre-position the cursor.
						switch (ei.getValue()) {
						case DataAccess.DA_WORDTOOLONG :
							// TODO: differentiate wordA and wordB.
							wordATxt.setFocus();
							break;
						default :		
							wordATxt.setFocus();
						}
						return;
					}
					// If no error, refresh lessons table.
					refreshPairsTable(dataAccess, pairsTbl);
				} else {
					// Update existing lesson.
					ErrorInfo ei = dataAccess.updatePair(previousWordA,
							                             previousWordB,
							                             wordATxt.getText(),
					                                     wordBTxt.getText());
					if (ei.getValue() != DataAccess.DA_SUCCESS) {
						MyMessageBox messageBox = new MyMessageBox(shell, SWT.OK);
						messageBox.setText(resReview.getString("ERR_error"));
						messageBox.setMessage(ei.getDescription());
						messageBox.open();
						// Stay in same state. And pre-position the cursor.
						switch (ei.getValue()) {
						case DataAccess.DA_WORDTOOLONG :
							// TODO: differentiate wordA and wordB.
							wordATxt.setFocus();
							break;
						default :		
							wordATxt.setFocus();
						}
						return;
					}
					// If no error, refresh lessons table.
					refreshPairsTable(dataAccess, pairsTbl);
				}
				//
				configureOnValidate();
			}
		});
		
		cancelBtn = new Button(buttonsC, SWT.PUSH);
		cancelBtn.setText(resReview.getString("MWW_cancel"));
		cancelBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		cancelBtn.setVisible(false);
		cancelBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				configureOnCancel();
			}
		});
		
		exitBtn = new Button(buttonsC, SWT.PUSH);
		exitBtn.setText(resReview.getString("MWW_exit"));
		exitBtn.setLayoutData(new RowData(BUTTON_LENGTH, BUTTON_HEIGHT));
		exitBtn.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				shell.dispose();
			}
		});

		
	}
	
	public void open() {
		
		shell.pack();
		shell.open(); 
		Display display = parent.getDisplay(); 
		while (!shell.isDisposed()) { 
			if (!display.readAndDispatch()) 
				display.sleep(); 
		}

	}


}
